import logging

import click

from deltalake_tools.__version__ import version
from deltalake_tools.core.core import (
    delta_compact,
    delta_create_checkpoint,
    delta_vacuum,
)
from deltalake_tools.core.core import table_version as delta_table_version
from deltalake_tools.core.convert import convert_parquet_to_delta
from deltalake_tools.models.models import (
    S3ClientDetails,
    S3KeyPairWrite,
    S3Scheme,
    TableType,
    VirtualAddressingStyle,
)

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version=version)
def cli() -> None:
    pass


@cli.command()
@click.argument("delta-table-path")
@click.option("--bucket", required=False, type=str)
@click.option("--access-key-id", required=False, type=str)
@click.option("--secret-access-key", required=False, type=str)
@click.option("--region", required=False, type=str)
@click.option("--endpoint-host", required=False, type=str)
@click.option("--port", type=int, required=False)
@click.option("--scheme", type=click.Choice(["http", "https"]), required=False)
@click.option("--allow-unsafe-https", is_flag=True)
@click.option("--path-addressing-style", is_flag=True)
def compact(delta_table_path: str, **kwargs) -> None:
    client_details = parse_cli_kwargs(delta_table_path, **kwargs)

    result = delta_compact(delta_table_path, client_details=client_details)

    if result.is_err():
        print(result.unwrap_err())
    else:
        print(result.unwrap())


@cli.command()
@click.argument("delta-table-path")
@click.option("--retention-hours", default=168, type=int)
@click.option("--disable-retention-duration", is_flag=True)
@click.option("--force", is_flag=True)
@click.option("--bucket", required=False, type=str)
@click.option("--access-key-id", required=False, type=str)
@click.option("--secret-access-key", required=False, type=str)
@click.option("--region", required=False, type=str)
@click.option("--endpoint-host", required=False, type=str)
@click.option("--port", type=int, required=False)
@click.option("--scheme", type=click.Choice(["http", "https"]), required=False)
@click.option("--allow-unsafe-https", is_flag=True)
@click.option("--path-addressing-style", is_flag=True)
def vacuum(
    delta_table_path: str,
    *,
    retention_hours: int,
    disable_retention_duration: bool = False,
    force: bool = False,
    **kwargs,
) -> None:
    client_details = parse_cli_kwargs(delta_table_path, **kwargs)
    result = delta_vacuum(
        delta_table_path,
        client_details=client_details,
        retention_hours=retention_hours,
        enforce_retention_duration=not disable_retention_duration,
        dry_run=not force,
    )

    if result.is_err():
        print(result.unwrap_err())
    else:
        print(result.unwrap())


@cli.command()
@click.argument("delta-table-path")
@click.option("--bucket", required=False, type=str)
@click.option("--access-key-id", required=False, type=str)
@click.option("--secret-access-key", required=False, type=str)
@click.option("--region", required=False, type=str)
@click.option("--endpoint-host", required=False, type=str)
@click.option("--port", type=int, required=False)
@click.option("--scheme", type=click.Choice(["http", "https"]), required=False)
@click.option("--allow-unsafe-https", is_flag=True)
@click.option("--path-addressing-style", is_flag=True)
def create_checkpoint(delta_table_path: str, **kwargs) -> None:
    client_details = parse_cli_kwargs(delta_table_path, **kwargs)

    result = delta_create_checkpoint(delta_table_path, client_details=client_details)

    if result.is_err():
        print(result.unwrap_err())
    else:
        print(result.unwrap())


@cli.command()
@click.argument("delta-table-path")
@click.option("--bucket", required=False, type=str)
@click.option("--access-key-id", required=False, type=str)
@click.option("--secret-access-key", required=False, type=str)
@click.option("--region", required=False, type=str)
@click.option("--endpoint-host", required=False, type=str)
@click.option("--port", type=int, required=False)
@click.option("--scheme", type=click.Choice(["http", "https"]), required=False)
@click.option("--allow-unsafe-https", is_flag=True)
@click.option("--path-addressing-style", is_flag=True)
def table_version(delta_table_path: str, **kwargs) -> None:
    client_details = parse_cli_kwargs(delta_table_path, **kwargs)

    result = delta_table_version(delta_table_path, client_details=client_details)

    if result.is_err():
        print(result.unwrap_err())
    else:
        print(result.unwrap())


def check_delta_table_type(delta_table_path: str) -> TableType:
    if delta_table_path.startswith("s3://"):
        return TableType.S3
    else:
        return TableType.Local


@cli.command()
@click.argument("table-path")
@click.option("--inplace", is_flag=True)
@click.option("--infer-partitioning", is_flag=True)
def parquet_to_delta(
    table_path: str, inplace: bool = False, infer_partitioning: bool = False
) -> None:

    # logger.warning(f"{table_path=}")
    # logger.warning(f"{inplace=}")
    # logger.warning(f"{infer_partitioning=}")

    result = convert_parquet_to_delta(
        table_path, inplace=inplace, infer_partitioning=infer_partitioning
    )

    if result.is_err():
        print(result.unwrap_err())
    else:
        print(result.unwrap())


def parse_cli_kwargs(
    delta_table_path,
    *,
    bucket: str,
    access_key_id: str,
    secret_access_key: str,
    region: str = "us-east-1",
    endpoint_host: str,
    port: int = 443,
    scheme: str = "https",
    allow_unsafe_https: bool = False,
    path_addressing_style: bool = VirtualAddressingStyle.Path,
) -> S3ClientDetails:
    client_details: S3ClientDetails = None
    # logger.error("in the table version command")
    table_type = check_delta_table_type(delta_table_path)
    if table_type == TableType.S3:
        if bucket is None:
            raise click.BadParameter(
                "Bucket must be provided, along with storage options (see -h), when using S3 table type."
            )
        # logger.error(f"Bucket: {bucket}")
        client_details_result = S3ClientDetails.default()
        if client_details_result.is_err():
            click.echo(client_details_result.unwrap_err())

        client_details = client_details_result.unwrap()

        if bucket is not None:
            client_details.bucket = bucket

        if access_key_id is not None and secret_access_key is not None:
            client_details.hmac_keys = S3KeyPairWrite(
                access_key_id=access_key_id, secret_access_key=secret_access_key
            )

        if region is not None:
            client_details.region = region

        if endpoint_host is not None:
            client_details.endpoint_host = endpoint_host

        if port is not None:
            client_details.port = port

        if scheme is not None:
            client_details.scheme = S3Scheme(scheme)

        if allow_unsafe_https:
            client_details.allow_unsafe_https = True

        if path_addressing_style:
            client_details.virtual_addressing_style = VirtualAddressingStyle.Path

    return client_details



import logging
import pyarrow.dataset as ds
import pyarrow as pa
from deltalake import convert_to_deltalake
from deltalake_tools.models.models import S3ClientDetails
from deltalake_tools.result import Err, Ok, Result

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class DeltaConverter:
    def __init__(self, source_path: str, table_path: str = None):
        self.source_path = source_path
        self.table_path = table_path

    def infer_partitioning(self) -> tuple[list[str], list[tuple[str, str]]]:
        dataset = ds.dataset(self.source_path, format="parquet", partitioning="hive")

        partitioning = dataset.partitioning

        if partitioning:
            schema = partitioning.schema
            partition_schema: list[tuple[str, str]] = []
            for field in schema:
                partition_schema.append((field.name, field.type))
            return pa.schema(partition_schema)

        else:
            print("The dataset is not partitioned.")
            return None

    def convert_parquet_to_delta(
        self,
        inplace: bool = False,
        storage_options: S3ClientDetails = None,
        partition_schema: pa.Schema = None,
        partition_strategy: str = None,
        infer_partitioning: bool = False,
    ) -> Result[str, str]:

        # logger.warning(f"{infer_partitioning=}")
        if infer_partitioning:
            partition_schema = self.infer_partitioning()

        if partition_schema is not None and partition_strategy is None:
            partition_strategy = "hive"

        if storage_options is not None:
            storage_options = storage_options.to_s3_config().unwrap()

        try:
            convert_to_deltalake(
                self.source_path,
                storage_options=storage_options,
                partition_by=partition_schema,
                partition_strategy=partition_strategy,
            )
        except Exception as e:
            logger.error(f"Error converting Parquet to Delta: {str(e)}")
            return Err(f"Error converting Parquet to Delta: {str(e)}")

        return Ok("Parquet converted to Delta successfully")


def convert_parquet_to_delta(
    source_path: str,
    table_path: str = None,
    inplace: bool = False,
    storage_options: S3ClientDetails = None,
    partition_schema: pa.Schema = None,
    infer_partitioning: bool = False,
) -> Result[str, str]:
    converter = DeltaConverter(source_path, table_path)
    return converter.convert_parquet_to_delta(
        inplace=inplace,
        storage_options=storage_options,
        partition_schema=partition_schema,
        infer_partitioning=infer_partitioning,
    )



# docs/conf.py

import os
import sys
import tomllib  # or import tomllib if using Python 3.11+

# -- Path setup --------------------------------------------------------------

# Add the project's root directory to sys.path
sys.path.insert(0, os.path.abspath('..'))

# -- Project information -----------------------------------------------------

# Read project metadata from pyproject.toml
with open("../pyproject.toml", "rb") as f:
    pyproject_data = tomllib.load(f)

project_info = pyproject_data.get("project", {})

project = project_info.get("name", "Your Project Name")
author = ", ".join([author["name"] for author in project_info.get("authors", [{"name": "Your Name"}])])
release = project_info.get("version", "0.1")
copyright = '2024, jeroenflvr'
# -- General configuration ---------------------------------------------------

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.todo',
    'sphinx.ext.githubpages',
]

templates_path = ['_templates']

exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

html_theme = 'sphinx_rtd_theme'

html_static_path = ['_static']

# -- Extension configuration -------------------------------------------------

# Autodoc settings
autodoc_member_order = 'bysource'

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True

# Todo extension settings
todo_include_todos = True



import logging
import tempfile
import os

import pandas as pd
import pytest
from deltalake import write_deltalake

from deltalake_tools.models.models import (S3ClientDetails, S3KeyPairWrite,
                                           S3Scheme, VirtualAddressingStyle,)

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@pytest.fixture(scope="session")
def s3key_pair_write() -> S3KeyPairWrite:
    return S3KeyPairWrite(
        access_key_id="test-access-key",
        secret_access_key="test-secret-access",
    )


@pytest.fixture(scope="session")
def s3_details(s3key_pair_write) -> S3ClientDetails:
    return S3ClientDetails(
        endpoint_host="localhost",
        region="us-east-1",
        virtual_addressing_style=VirtualAddressingStyle.Path,
        port=5002,
        bucket="my-test-bucket",
        hmac_keys=s3key_pair_write,
        scheme=S3Scheme.Http,
        allow_unsafe_https=True,
    )


@pytest.fixture(scope="session")
def tmp_path():
    tmp_path = tempfile.TemporaryDirectory()
    yield tmp_path.name

@pytest.fixture(scope="session")
def tmp_path_partitioned():
    tmp_path = tempfile.TemporaryDirectory()
    yield tmp_path.name

@pytest.fixture(scope="session")
def tmp_output_path_partitioned():
    tmp_path = tempfile.TemporaryDirectory()
    yield tmp_path.name

@pytest.fixture(scope="session")
def cli_tmp_path_partitioned():
    tmp_path = tempfile.TemporaryDirectory()
    yield tmp_path.name
    tmp_path.cleanup()

@pytest.fixture(scope="session")
def tmp_output_path():
    tmp_path = tempfile.TemporaryDirectory()
    yield tmp_path.name


@pytest.fixture(scope="session")
def parquet_table_path(tmp_path):
    parquet_table = f"{tmp_path}/parquet-table"
    os.makedirs(parquet_table)
    parquet_filename = f"{parquet_table}/data.parquet"

    data = pd.DataFrame({"id": [1, 2], "name": ["Alice", "Bob"]})

    data.to_parquet(parquet_filename)

    yield parquet_table

@pytest.fixture(scope="session")
def partitioned_parquet_table_path(tmp_path_partitioned):
    parquet_table = f"{tmp_path_partitioned}/parquet-table"

    data = pd.DataFrame({"id": [1, 2, 3], 
                         "name": ["Alice", "Bob", "Frank"],
                         "job": ["Engineer", "Doctor", "Doctor"]}
                    )

    data['job'] = data['job'].astype('category')
    data.to_parquet(parquet_table, partition_cols=["job"])

    yield parquet_table

@pytest.fixture(scope="session")
def cli_partitioned_parquet_table_path(cli_tmp_path_partitioned):
    parquet_table = f"{cli_tmp_path_partitioned}/parquet-table"

    data = pd.DataFrame({"id": [1, 2, 3], 
                         "name": ["Alice", "Bob", "Frank"],
                         "job": ["Engineer", "Doctor", "Doctor"]}
                    )

    data['job'] = data['job'].astype('category')
    data.to_parquet(parquet_table, partition_cols=["job"])

    yield parquet_table

@pytest.fixture(scope="session")
def delta_table_path(tmp_path):
    delta_table_path = f"{tmp_path}/delta-table"

    data = pd.DataFrame({"id": [1, 2], "name": ["Alice", "Bob"]})

    write_deltalake(delta_table_path, data, mode="overwrite")

    yield delta_table_path


@pytest.fixture(scope="session")
def cli_delta_table_path(tmp_path):
    delta_table_path = f"{tmp_path}/delta-table_cli"

    data = pd.DataFrame({"id": [1, 2], "name": ["Alice", "Bob"]})

    write_deltalake(delta_table_path, data, mode="overwrite")

    yield delta_table_path


# @pytest.fixture(scope="session")
# def delta_table_with_data(tmp_path):
#     # Create a temporary directory for the Delta table
#     delta_table_path = (f"{tmp_path.name}/delta-table")

#     # Initialize the Delta table with some mock data
#     for i in range(10):
#         data = pd.DataFrame({
#             "id": [i+2, i+3],
#             "name": [f"{str(i)}_name", f"{str(i)}_other_name"]
#         })

#         write_deltalake(delta_table_path,
#                         data,
#                         mode="append"
#                     )
#     # Write the data to the Delta table
#     # write_deltalake(delta_table_path, data)

#     yield delta_table_path

# @pytest.fixture(scope="session")
# def s3_delta_table_path(s3_bucket, s3_details):
#     delta_table_path = f"s3://{s3_bucket}/cli-delta-table"

#     storage_options_result = s3_details.to_s3_config()
#     assert storage_options_result.is_ok()

#     storage_options = storage_options_result.unwrap()
#     # logger.error(f"{storage_options=}")

#     for i in range(10):
#         data = pd.DataFrame({
#             "id": [i+2, i+3],
#             "name": [f"{str(i)}_name", f"{str(i)}_other_name"]
#         })

#         write_deltalake(delta_table_path,
#                         data,
#                         mode="overwrite",
#                         storage_options=storage_options
#                     )

#     write_deltalake(delta_table_path, data, mode="append", storage_options=storage_options)

#     assert delta_table_path
#     dt = DeltaTable(delta_table_path, storage_options=storage_options)

#     assert dt.version() == 10

#     yield delta_table_path



import logging
from time import sleep
import pandas as pd
import pytest
from click.testing import CliRunner
from deltalake import DeltaTable, write_deltalake

from deltalake_tools.cli.cli import (
    compact,
    create_checkpoint,
    table_version,
    vacuum,
    parquet_to_delta,
)

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@pytest.fixture(scope="session")
def cli_delta_table_with_data(cli_delta_table_path):
    dt = DeltaTable(cli_delta_table_path)
    # logger.error(f"{cli_delta_table_path=}")

    assert dt.version() == 0

    for i in range(10):
        data = pd.DataFrame(
            {"id": [i + 2, i + 3], "name": [f"{str(i)}_name", f"{str(i)}_other_name"]}
        )

        write_deltalake(cli_delta_table_path, data, mode="append")

    dt_result = DeltaTable(cli_delta_table_path)

    assert dt_result.version() == 10
    assert len(dt_result.to_pandas()) == 22

    assert len(dt_result.files()) == 11

    # shutil.copytree(cli_delta_table_path, "/tmp/delta_table_backup")

    yield cli_delta_table_path


# @pytest.fixture(scope="session")
# def parquet_table_with_data(cli_tmp_path_partitioned):
#     pass


def test_compact(cli_delta_table_with_data):
    runner = CliRunner()
    result = runner.invoke(compact, [cli_delta_table_with_data])
    assert result.exit_code == 0
    assert "numFilesAdded" in result.output


def test_vacuum(cli_delta_table_with_data, cli_delta_table_path):
    runner = CliRunner()
    result = runner.invoke(
        vacuum,
        [
            cli_delta_table_with_data,
            "--retention-hours",
            0,
            "--force",
            "--disable-retention-duration",
        ],
    )
    assert result.exit_code == 0
    assert "parquet" in result.output


def test_create_checkpoint(cli_delta_table_with_data, cli_delta_table_path):
    runner = CliRunner()
    result = runner.invoke(create_checkpoint, [cli_delta_table_with_data])

    assert result.exit_code == 0
    assert "Checkpoint created successfully" in result.output


def test_table_version(cli_delta_table_with_data):
    runner = CliRunner()
    result = runner.invoke(table_version, [cli_delta_table_with_data])

    assert result.exit_code == 0
    assert result.output.strip().isdigit()


@pytest.mark.run
def test_convert_parquet_to_delta(cli_partitioned_parquet_table_path, capsys):

    with capsys.disabled():
        logger.warning(f"cli test: {cli_partitioned_parquet_table_path=}")
        runner = CliRunner()

        result = runner.invoke(
            parquet_to_delta,
            args=[
                cli_partitioned_parquet_table_path,
                "--infer-partitioning",
            ],
            catch_exceptions=False,
        )

        assert result.exit_code == 0
        assert result.output.strip() == "Parquet converted to Delta successfully"



import logging

from deltalake_tools.result import Err, Ok, Result

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def divide(a: float, b: float) -> Result[float, str]:
    if b == 0:
        return Err("Division by zero")
    else:
        return Ok(a / b)


def evaluate_result(result: Result) -> None:
    if result.is_ok():
        logger.info(f"Result: {result.unwrap()}")
    else:
        logging.info(f"Error: {result.unwrap_err()}")


def test_result() -> None:
    result = divide(10, 2)
    assert result.is_ok()
    evaluate_result(result)

    error_result = divide(10, 0)
    assert error_result.is_err()
    evaluate_result(error_result)



from deltalake_tools.models.models import (S3ClientDetails, S3Scheme,
                                           VirtualAddressingStyle,)


def test_s3_details_default() -> None:
    default_result = S3ClientDetails.default()
    # default_result = s3_details.default()
    assert default_result.is_ok()
    assert default_result.unwrap() == S3ClientDetails(
        endpoint_host="s3.amazonaws.com",
        region="us-east-1",
        addressing_style=VirtualAddressingStyle.Virtual,
        port=443,
        bucket=None,
        hmac_keys=None,
        scheme=S3Scheme.Https,
    )


def test_s3_config(s3_details: S3ClientDetails) -> None:
    config_result = s3_details.to_s3_config()
    assert config_result.is_ok()
    config = config_result.unwrap()

    assert (
        config["aws_virtual_hosted_style_request"]
        == s3_details.virtual_addressing_style.value
    )
    assert (
        config["endpoint"]
        == f"{s3_details.scheme.value}://{s3_details.endpoint_host}:{s3_details.port}"
    )
